#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <com_err.h>
#include <time.h>
#include "owl.h"


void owl_function_adminmsg(char *header, char *body) {
  owl_message *m;
  
  m=malloc(sizeof(owl_message));
  owl_message_create_admin(m, header, body);
  owl_messagelist_append_element(owl_global_get_msglist(&g), m);

  owl_mainwin_redisplay(owl_global_get_mainwin(&g));
  if (owl_popwin_is_active(owl_global_get_popwin(&g))) {
    owl_popwin_refresh(owl_global_get_popwin(&g));
  }
  
  wnoutrefresh(owl_global_get_curs_recwin(&g));
  owl_global_set_needrefresh(&g);
}


void owl_function_nextmsg() {
  int curmsg;
  curmsg=owl_global_get_curmsg(&g);
  curmsg++;
  if (curmsg>owl_messagelist_get_size(owl_global_get_msglist(&g))-1) {
    curmsg=owl_messagelist_get_size(owl_global_get_msglist(&g))-1;
    if (curmsg<0) curmsg=0;
    owl_function_beep();
    owl_function_makemsg("already at last message");
  }
  owl_global_set_curmsg(&g, curmsg);
  owl_function_calculate_topmsg(OWL_DIRECTION_DOWNWARDS);

  owl_mainwin_redisplay(owl_global_get_mainwin(&g));
  owl_global_set_direction_downwards(&g);
}


void owl_function_prevmsg() {
  int curmsg;
  curmsg=owl_global_get_curmsg(&g);
  curmsg--;
  if (curmsg<0) {
    curmsg=0;
    owl_function_beep();
    owl_function_makemsg("already at first message");
  }
  owl_global_set_curmsg(&g, curmsg);
  owl_function_calculate_topmsg(OWL_DIRECTION_UPWARDS);
  owl_mainwin_redisplay(owl_global_get_mainwin(&g));
  owl_global_set_direction_upwards(&g);
}


void owl_function_nextmsg_notdeleted() {
  int curmsg;
  owl_message *m;

  curmsg=owl_global_get_curmsg(&g);
  while (1) {
    curmsg++;

    /* if we're out of bounds get in bounds and stop */
    if (curmsg>owl_messagelist_get_size(owl_global_get_msglist(&g))-1) {
      curmsg=owl_messagelist_get_size(owl_global_get_msglist(&g))-1;
      if (curmsg<0) curmsg=0;
      owl_function_makemsg("already at last non-deleted message");
      break;
    }

    /* if this one is not deleted we can stop where we are */
    m=owl_messagelist_get_element(owl_global_get_msglist(&g), curmsg);
    if (!owl_message_is_delete(m)) {
      break;
    }
  }
  
  owl_global_set_curmsg(&g, curmsg);
  owl_function_calculate_topmsg(OWL_DIRECTION_DOWNWARDS);
  owl_mainwin_redisplay(owl_global_get_mainwin(&g));
  owl_global_set_direction_downwards(&g);
}


void owl_function_prevmsg_notdeleted() {
  int curmsg;
  owl_message *m;

  curmsg=owl_global_get_curmsg(&g);
  while(1) {
    curmsg--;

    /* if we're out of bounds get in bounds and stop */
    if (curmsg<0) {
      curmsg=0;
      owl_function_makemsg("already at first non-deleted message");
      break;
    }

    /* if this one is not deleted we can stop where we are */
    m=owl_messagelist_get_element(owl_global_get_msglist(&g), curmsg);
    if (!owl_message_is_delete(m)) {
      break;
    }
  }

  owl_global_set_curmsg(&g, curmsg);
  owl_function_calculate_topmsg(OWL_DIRECTION_UPWARDS);
  owl_mainwin_redisplay(owl_global_get_mainwin(&g));
  owl_global_set_direction_upwards(&g);
}


void owl_function_deletecur() {
  int curmsg;

  /* bail if there's no current message */
  if (owl_messagelist_get_size(owl_global_get_msglist(&g)) < 1) {
    owl_function_makemsg("No current message to delete");
    return;
  }

  /* mark the message for deletion */
  curmsg=owl_global_get_curmsg(&g);
  owl_messagelist_delete_element(owl_global_get_msglist(&g), curmsg);

  /* move the poiner in the appropriate direction to the next undeleted msg */
  if (owl_global_get_direction(&g)==OWL_DIRECTION_UPWARDS) {
    owl_command_prev_notdeleted();
  } else {
    owl_command_next_notdeleted();
  }
}


void owl_function_undeletecur() {
  int curmsg;
  
  if (owl_messagelist_get_size(owl_global_get_msglist(&g)) < 1) {
    owl_function_makemsg("No current message to undelete");
    return;
  }
  curmsg=owl_global_get_curmsg(&g);

  owl_messagelist_undelete_element(owl_global_get_msglist(&g), curmsg);

  if (owl_global_get_direction(&g)==OWL_DIRECTION_UPWARDS) {
    if (curmsg>0) {
      owl_command_prev();
    } else {
      owl_command_next();
    }
  } else {
    owl_command_next();
  }

  owl_mainwin_redisplay(owl_global_get_mainwin(&g));
}


void owl_function_expunge() {
  int curmsg;

  owl_messagelist_expunge(owl_global_get_msglist(&g));

  curmsg=owl_global_get_curmsg(&g);
  if (curmsg>owl_messagelist_get_size(owl_global_get_msglist(&g))-1) {
    owl_global_set_curmsg(&g, owl_messagelist_get_size(owl_global_get_msglist(&g))-1);
    if (owl_global_get_curmsg(&g)<0) {
      owl_global_set_curmsg(&g, 0);
    }
    owl_function_calculate_topmsg(OWL_DIRECTION_NONE);
  }

  /* if there are no messages set the direction to down in case we
     delete everything upwards */
  owl_global_set_direction_downwards(&g);
  
  owl_function_makemsg("Messages expunged");
  owl_mainwin_redisplay(owl_global_get_mainwin(&g));
}


void owl_function_firstmsg() {
  owl_global_set_curmsg(&g, 0);
  owl_global_set_topmsg(&g, 0);
  owl_mainwin_redisplay(owl_global_get_mainwin(&g));
  owl_global_set_direction_downwards(&g);
}


void owl_function_lastmsg() {
  int curmsg;
  
  curmsg=owl_messagelist_get_size(owl_global_get_msglist(&g))-1;
  if (curmsg<0) curmsg=0;
  owl_global_set_curmsg(&g, curmsg);
  owl_global_set_topmsg(&g, curmsg);
  owl_mainwin_redisplay(owl_global_get_mainwin(&g));
  owl_global_set_direction_upwards(&g);
}


void owl_function_shift_right() {
  owl_global_set_rightshift(&g, owl_global_get_rightshift(&g)+10);
  owl_mainwin_redisplay(owl_global_get_mainwin(&g));
  owl_global_set_needrefresh(&g);
}


void owl_function_shift_left() {
  int shift;

  shift=owl_global_get_rightshift(&g);
  if (shift>=10) {
    owl_global_set_rightshift(&g, shift-10);
    owl_mainwin_redisplay(owl_global_get_mainwin(&g));
    owl_global_set_needrefresh(&g);
  } else {
    owl_function_beep();
    owl_function_makemsg("Already full left");
  }
}


void owl_function_unsub() {
  unsuball();
  owl_function_makemsg("Unsubscribed from all messages.");
}


void owl_function_loadsubs(char *file) {
  int ret;
  ret=loadsubs(file);
  if (ret==0) {
    owl_function_makemsg("Subscribed to messages from file.");
  } else {
    owl_function_makemsg("Error subscribing to messages from file.");
  }
}


void owl_function_suspend() {
  endwin();
  printf("\n");
  kill(getpid(), SIGSTOP);

  /* resize to reinitialize all the windows when we come back */
  owl_command_resize();
}


void owl_function_zaway_on(char *msg) {
  char buff[1024];
  /* turn on zaway.  If msg!=NULL then
     use msg as the zaway message */
  if (msg) {
    owl_global_set_zaway_msg(&g, msg);
  } else {
    owl_global_set_zaway_msg_default(&g);
  }
  owl_global_set_zaway_on(&g);
  owl_global_set_needrefresh(&g);
  sprintf(buff, "zaway set (%s)", owl_global_get_zaway_msg(&g));
  owl_function_makemsg(buff);
}


void owl_function_zaway_off() {
  owl_global_set_zaway_off(&g);
  owl_function_makemsg("zaway off");
}


void owl_function_quit() {
  /* zlog out if we need to */
  if (owl_global_get_shutdownlogout(&g)) {
    owl_function_zlog_out();
  }

  /* execute the commands in shutdown */
  owl_config_execute("owl::shutdown();");

  /* final clean up */
  unsuball();
  ZClosePort();
  endwin();
  owl_function_debugmsg("Quitting Owl");
  exit(0);
}


void owl_function_zlog_in() {
  char *exposure, *eset;
  int ret;
  char buff[LINE];

  eset=EXPOSE_REALMVIS;
  exposure=ZGetVariable("exposure");
  if (exposure==NULL) {
    eset=EXPOSE_REALMVIS;
  } else if (!strcasecmp(exposure,EXPOSE_NONE)) {
    eset = EXPOSE_NONE;
  } else if (!strcasecmp(exposure,EXPOSE_OPSTAFF)) {
    eset = EXPOSE_OPSTAFF;
  } else if (!strcasecmp(exposure,EXPOSE_REALMVIS)) {
    eset = EXPOSE_REALMVIS;
  } else if (!strcasecmp(exposure,EXPOSE_REALMANN)) {
    eset = EXPOSE_REALMANN;
  } else if (!strcasecmp(exposure,EXPOSE_NETVIS)) {
    eset = EXPOSE_NETVIS;
  } else if (!strcasecmp(exposure,EXPOSE_NETANN)) {
    eset = EXPOSE_NETANN;
  }
   
  ret=ZSetLocation(eset);
  if (ret) {
    sprintf(buff, "Error setting location: %s", error_message(ret));
    owl_function_makemsg(buff);
  }
}

void owl_function_zlog_out() {
  int ret;
  char buff[LINE];
  
  ret=ZUnsetLocation();
  if (ret) {
    sprintf(buff, "Error unsetting location: %s", error_message(ret));
    owl_function_makemsg(buff);
  }
}


void owl_function_makemsg(char *in) {
  werase(owl_global_get_curs_msgwin(&g));
  waddstr(owl_global_get_curs_msgwin(&g), in);
  wnoutrefresh(owl_global_get_curs_msgwin(&g));
  owl_global_set_needrefresh(&g);
}


void owl_function_openurl() {
  /* visit the first url in the current message */
  owl_message *m;
  char *ptr1, *ptr2, *text, url[LINE], tmpbuff[LINE];

  if (owl_messagelist_get_size(owl_global_get_msglist(&g))==0) {
    owl_function_makemsg("No current message selected");
    return;
  }

  m=owl_messagelist_get_element(owl_global_get_msglist(&g),
				owl_global_get_curmsg(&g));
  text=owl_message_get_text(m);

  /* First look for a good URL */  
  if ((ptr1=strstr(text, "http://"))!=NULL) {
    ptr2=strpbrk(ptr1, " \n\t");
    if (ptr2) {
      strncpy(url, ptr1, ptr2-ptr1+1);
      url[ptr2-ptr1+1]='\0';
    } else {
      strcpy(url, ptr1);
    }

    /* if we had <http strip a trailing > */
    if (ptr1>text && ptr1[-1]=='<') {
      if (url[strlen(url)-1]=='>') {
	url[strlen(url)-1]='\0';
      }
    }
  } else if ((ptr1=strstr(text, "https://"))!=NULL) {
    /* Look for an https URL */  
    ptr2=strpbrk(ptr1, " \n\t");
    if (ptr2) {
      strncpy(url, ptr1, ptr2-ptr1+1);
      url[ptr2-ptr1+1]='\0';
    } else {
      strcpy(url, ptr1);
    }
    
    /* if we had <http strip a trailing > */
    if (ptr1>text && ptr1[-1]=='<') {
      if (url[strlen(url)-1]=='>') {
	url[strlen(url)-1]='\0';
      }
    }
  } else if ((ptr1=strstr(text, "www."))!=NULL) {
    /* if we can't find a real url look for www.something */
    ptr2=strpbrk(ptr1, " \n\t");
    if (ptr2) {
      strncpy(url, ptr1, ptr2-ptr1+1);
      url[ptr2-ptr1+1]='\0';
    } else {
      strcpy(url, ptr1);
    }
  } else {
    owl_function_beep();
    owl_function_makemsg("Could not find URL to open.");
    return;
  }
  
  /* open the page */
  sprintf(tmpbuff, "Opening %s ...", url);
  owl_function_makemsg(tmpbuff);
  sprintf(tmpbuff, "netscape -remote \"openURL(%s)\" > /dev/null 2> /dev/null", url);
  system(tmpbuff);
}

void owl_function_calculate_topmsg(int direction) {
  int recwinlines, y, savey, i, j, topmsg, curmsg, foo;
  owl_messagelist *msglist;
  owl_mainwin *mw;

  mw=owl_global_get_mainwin(&g);

  msglist=owl_global_get_msglist(&g);
  topmsg=owl_global_get_topmsg(&g);
  curmsg=owl_global_get_curmsg(&g);
  recwinlines=owl_global_get_recwin_lines(&g);

  if (owl_messagelist_get_size(owl_global_get_msglist(&g)) < 1) {
    return;
  }
  
  /* Find number of lines from top to bottom of curmsg (store in savey) */
  savey=0;
  for (i=topmsg; i<=curmsg; i++) {
    savey+=owl_message_get_numlines(owl_messagelist_get_element(msglist, i));
  }

  /* If our bottom line is less than 1/4 down the screen then scroll up */
  if (direction == OWL_DIRECTION_UPWARDS || direction == OWL_DIRECTION_NONE) {
    if (savey < (recwinlines / 4)) {
      y=0;
      for (j=curmsg; j>0; j--) {
	foo=owl_message_get_numlines(owl_messagelist_get_element(msglist, j));
	/* will we run the curmsg off the screen? */
	if ((foo+y) >= recwinlines) {
	  j++;
	  if (j>curmsg) j=curmsg;
	  break;
	}
	/* have saved 1/2 the screen space? */
	y+=foo;
	if (y > (recwinlines / 2)) break;
      }
      owl_global_set_topmsg(&g, j);
      return;
    }
  }

  /* If our bottom line is more than 3/4 down the screen then scroll down */
  if (direction == OWL_DIRECTION_DOWNWARDS || direction == OWL_DIRECTION_NONE) {
    if (savey > ((recwinlines * 3)/4)) {
      y=0;
      /* count lines until we can save 1/2 the screen size */
      for (j=topmsg; j<curmsg; j++) {
	y+=owl_message_get_numlines(owl_messagelist_get_element(msglist, j));
	if (y > (recwinlines / 2)) break;
      }
      if (j==curmsg) {
	j--;
      }
      owl_global_set_topmsg(&g, j+1);
      return;
    }
  }
}


void owl_function_resize() {
  owl_global_set_resize_pending(&g);
}

void owl_function_resize_typwin(int newsize) {
  owl_global_set_typwin_lines(&g, newsize);
  owl_function_resize();
}

void owl_function_run_buffercommand() {
  char *buff;

  buff=owl_global_get_buffercommand(&g);
  if (!strncmp(buff, "zwrite ", 7)) {

    owl_command_zwrite(buff);
  }
}

void owl_function_debugmsg(char *in) {
  FILE *file;
  time_t now;
  char buff1[LINE], buff2[LINE];
  

  if (!owl_global_is_debug(&g)) return;

  file=fopen(OWL_DEBUG_FILE, "a");
  if (!file) return;

  now=time(NULL);
  strcpy(buff1, ctime(&now));
  buff1[strlen(buff1)-1]='\0';

  owl_global_get_runtime(&g, buff2);
  
  fprintf(file, "[%i -  %s - %s]: %s\n", getpid(), buff1, buff2, in);
  fclose(file);
}


void owl_function_refresh() {
  owl_function_resize();
}

void owl_function_beep() {
  if (owl_global_is_bell(&g)) {
    beep();
  }
}
